/*
  Melody_picaxe
  
  Plays a melody using Picaxe tune command format

  circuit:
  - 8 ohm speaker and 100 ohm resistor on digital pin 8
  or piezo-speaker on digital pin 8

  created 22 July 2019
  by Dmitriy Korabelnikov aka Deckard of Atomic Destruction
  e-mail:
  d_korabelnikov@mail.ru
  site:
  http://atd-group.narod.ru
*/  

//You can play with this value. Write 1 or 0, then look at the size of compiled code.
#define SHORTCODE 1
// notes.h contains melody
// Use ringtv.exe and ringxmas.exe to select melody
#include "notes.h"
//Octave frequancy table for 12 notes (including with # notes)
const PROGMEM unsigned int freqtab[]={200,211,224,238,252,267,283,300,318,337,358,379};
//Length of 1/8 note in milliseconds
//Possible values: from 125 (fast) to 250 (slow)
#define SPEED 125
//Place your beeper pin number here
const int buzzer = 8;

const unsigned char coct[]={1,2,0,1}; //note octaves
const unsigned char clen[]={2,1,8,4}; //note durations: 1/4, 1/8, 1, 1/2
unsigned int freq,durat;

#if SHORTCODE
void toneWithDelay (uint8_t _pin, unsigned int frequency, unsigned long duration)
{
  unsigned long TimeMs, TimeUs;
  unsigned int Micros;
  char PinValue;
  
  PinValue = 0;
  TimeMs = millis();
  Micros = 500000L / frequency;
  while (!((millis() - TimeMs) >= duration)) {
    TimeUs = micros();
    while (!((micros() - TimeUs) >= Micros));
    PinValue = ~PinValue;
    if (frequency) digitalWrite(_pin, PinValue);
  }
  digitalWrite(_pin, 0);
}
#else
//This version looks short, but produce a very big code (+about 1 kilobyte)
void toneWithDelay(uint8_t _pin, unsigned int frequency, unsigned long duration)
{tone(_pin,frequency,duration);delay(duration);}
#endif

void decod(unsigned char d)
{unsigned char note,ioctave,ilen;
 note=d & 15;ioctave=(d >> 4) & 3;ilen=(d >> 6) & 3;
 durat=SPEED*(unsigned int)(clen[ilen]);
 if (note != 12) freq=pgm_read_word(freqtab+note)  << coct[ioctave];else freq=0;
}

void play(const unsigned char *melody)
{unsigned char d;unsigned int i;
 for (i=0;d=pgm_read_byte(melody+i), d != 255;i++) {decod(d);toneWithDelay(buzzer,freq,durat);}
}

void setup()
{
 pinMode(buzzer,OUTPUT);play(notes);
}

void loop()
{}

